/*******************************************************************************
 * Example for EMG click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration FT90x :
 *  MCU           :        FT900
 *  Dev. Board    :        EasyFT90x
 *  SW            :        mikroC PRO for FT90x v2.0.0
 *
 *  NOTES         :
 *                   - Place EMG click board on mikroBUS slot 1
 *                   - Turn switches SW2.2 and SW2.3 in right position
 *                   - Use USB UART
 *                   - Pull-up switch SW1.2 (VCC position) and
 *                   - Pull-down GPIO_PORT_08_15.B0 at SW21.8.
 *                     ( Use on-click potentiometer for signal calibration )
 *                     ( Set COM port Latency Timer to 1 ms )
*******************************************************************************/

#include <stdint.h>
#include <stdbool.h>

void InitTimerA() {
  // 512 Hz / 1.95 ms
  TIMER_CONTROL_0 = 2;
  // Select Timer A
  TIMER_SELECT = 0;
  // Write prescaler for Timer A
  TIMER_PRESC_LS = 232;
  TIMER_PRESC_MS = 253;
  // Write start value
  TIMER_WRITE_LS = 2;
  TIMER_WRITE_MS = 0;
  // Set continuous mode and direction down
  TIMER_CONTROL_3 = 0;
  // Trigger clearing timer A and prescaler
  TIMER_CONTROL_4 = 0x11;
  // Enable prescaler for Timer A
  TIMER_CONTROL_2 = 0x10;
  // Enable Timer A interrupt
  TIMER_INT.B1 = 1;
  // Enable global interrupts
  IRQ_CTRL.B31 = 0;
  // Start timer A
  TIMER_CONTROL_1 = 1;
}

static bool read_flag = false;
static volatile uint32_t interrupt_ctr = 0;
uint32_t adc_reads_ctr = 0;
uint32_t timer_reads_ctr = 0;
static volatile uint32_t seconds_counter = 0;
uint32_t temp_adc_read;
double temp_timer_read;

void main()
{
     uint32_t i = 0;
     char timer_read_string[10];
     char final_string [20];

     UART1_Init(57600);
     delay_ms(500);

     ADC_Init_Advanced(_ADC_MODE_SINGLE);
     ADC_Set_Input_Channel(_ADC_CHANNEL_4);
     Delay_ms(100);
     GPIO_Digital_Input(&GPIO_PORT_08_15, _GPIO_PINMASK_0);         // set PIN 8 input for button press

     while(1)
     {
        // if the button PIN8 is pressed, the measuring begins
        if (Button(&GPIO_PORT_08_15, 0, 10, 1))        
        {
          UART1_Write_Text("START\r\n");
          delay_ms(500);
          InitTimerA();
        }

        // Every 1.95 ms measure data and send them to mikroPlot
        if (read_flag == true)                        
        {
              read_flag = false;

              // Disable global interrupts
              IRQ_CTRL.B31 = 1;
        
              temp_adc_read =  ADC_Read(4);
            
              temp_timer_read  =  ((double)interrupt_ctr) * 1.95;
              inttostr(temp_adc_read, final_string);
              sprintf(timer_read_string,"%.2f", temp_timer_read);
              strcat(final_string, ",");
              strcat(final_string, timer_read_string);
              ltrim(final_string);
              UART1_Write_Text(final_string);
              UART1_Write_Text("\r\n");

              // Enable global interrupts
              IRQ_CTRL.B31 = 0;       
        } 
     }

}


void TimerInterrupt() iv IVT_TIMERS_IRQ
{

  if (TIMER_INT_A_bit)
  {
    read_flag = true;
    interrupt_ctr++;
    if (interrupt_ctr % 300 == 0)
           seconds_counter++;

    TIMER_INT = (TIMER_INT & 0xAA) | (1 << 0);
  }
}
/************************************************************ END OF FILE *****/